% 'Matchine' software by Joan Alexis Glaunes, Copyright (C) Universite Paris Descartes, All Rights Reserved 2005-2006, version 1.0
%
% Any use of this code should make reference to at least one of the following references:
%
% - [for the use of 'measure matching']:
% J. Glaunes, L. Younes and A. Trouve, Diffeomorphic matching of distributions: A new approach for unlabelled point-sets and sub-manifolds matching, Proc. of the 2004 IEEE Computer Society Conference on Computer Vision and Pattern Recognition. (CVPR'04), Vol. 2, pp. 712--718, DOI: 10.1109/CVPR.2004.81
%
% - [for the use of 'curve matching']:
% J. Glaunes, A. Qiu, M. Miller, L. Younes, Large Deformation Diffeomorphic Metric Curve Mapping, International Journal of Computer vision, Springer, 2008, Vol. 80, No. 3, pp. 317--336, DOI: 10.1007/s11263-008-0141-9
%
% - [for the use of 'surface matching']:
% M. Vaillant and J. Glaunes, Surface Matching via Currents, Proc. of Information Processing in Medical Imaging (IPMI'05), Lecture Notes in Computer Science vol. 3565, Springer 2005, pp. 381--392


function [x,J] = adaptdesc(x,f,g,options,varargin)

% Steepest descent with adaptive stepsize
%
% f functional
% g gradient of f
% x variable
% in structure options:
%   maxiter = maximum number of iterations; default 500;
%   stepsize = initial stepsize of descent; default 1/norm(initial gradient)
%   stepmult = increase ratio of stepsize after each minimization success;
%     default 1.2
%   stepdiv = decrease ratio of stepsize after each minimization failure;
%     default 2
%   breakratio = ratio for termination criteria; default 1e-4
%   loopbreak = maximum number of loops in each iteration
%   verbosemode if 1 then display evaluation of f at each step
%
% varargin additional arguments passed to f and g
%
% returns x variable at the end of minimization
% and J record of function evaluations
%
% contact : Joan Glauns - http://www.mi.parisdescartes.fr/~glaunes/

maxiter = 500;
J(1) = feval(f,x,varargin{:});
G = feval(g,x,varargin{:});
stepsize = J(1)/sum(G(:).^2);
stepmult = 1.2;
stepdiv = 2;
breakratio = 1e-4;
loopbreak = 40;
verbosemode = 0;

loadstruct('options');

for iter = 1:maxiter
    G = feval(g,x,varargin{:});
    stepsize = stepsize * stepdiv;
    minimtest = 0;
    loop = 0;
    while(~minimtest & (loop<loopbreak)) 
        stepsize = stepsize / stepdiv;
        xnew = x - stepsize * G; 
        J(iter+1) = feval(f,xnew,varargin{:});
        minimtest = (J(iter+1) < J(iter));
        loop = loop + 1;
    end
    x = xnew;
    stepsize = stepsize * stepmult;
    if verbosemode
        disp(['iteration ',num2str(iter),...
            '     functional = ',num2str(J(iter)),...
            '     stepsize = ', num2str(stepsize)]);
    end
    if dosaveiter
        feval(saveiter,x,J)
    end
    % termination criteria
    if (J(iter)-J(iter+1)) < breakratio*(J(1)-J(iter+1)) || loop==loopbreak
        return
    end
end

if verbosemode
    disp('maximum number of iterations exceeded');
end
