% 'Matchine' software by Joan Alexis Glaunes, Copyright (C) Universite Paris Descartes, All Rights Reserved 2005-2006, version 1.0
%
% Any use of this code should make reference to at least one of the following references:
%
% - [for the use of 'measure matching']:
% J. Glaunes, L. Younes and A. Trouve, Diffeomorphic matching of distributions: A new approach for unlabelled point-sets and sub-manifolds matching, Proc. of the 2004 IEEE Computer Society Conference on Computer Vision and Pattern Recognition. (CVPR'04), Vol. 2, pp. 712--718, DOI: 10.1109/CVPR.2004.81
%
% - [for the use of 'curve matching']:
% J. Glaunes, A. Qiu, M. Miller, L. Younes, Large Deformation Diffeomorphic Metric Curve Mapping, International Journal of Computer vision, Springer, 2008, Vol. 80, No. 3, pp. 317--336, DOI: 10.1007/s11263-008-0141-9
%
% - [for the use of 'surface matching']:
% M. Vaillant and J. Glaunes, Surface Matching via Currents, Proc. of Information Processing in Medical Imaging (IPMI'05), Lecture Notes in Computer Science vol. 3565, Springer 2005, pp. 381--392



function s = shooting(s)

% geodesic shooting for point matching

T = s.T;
tau = 1/(T-1);
nx = s.nx;
X = zeros(3,nx,T);
mom = zeros(3,nx,T);
X(:,:,1) = s.transmatrix * s.x + repmat(s.transvector,1,nx);
mom(:,:,1) = s.mom(:,:,1);
sigmaV = s.sigmaV;
sigmaV2 = sigmaV.^2;
usefgt = s.usefgt;
order = s.order;
K = s.K;
e = s.e;

dX = zeros(3,nx);
dmom = zeros(3,nx);

for t=1:T-1
    kernelsum(t);
    dkernelsum(t);

    X(:,:,t+1) = X(:,:,t) + tau * dX;
    mom(:,:,t+1) = mom(:,:,t) + (tau * 2 / sigmaV2(t)) * dmom;

    kernelsum(t+1);
    dkernelsum(t+1);
    
    dX = .5 * dX;
    dmom = .5 * dmom;

    X(:,:,t+1) = X(:,:,t) + tau * dX;
    mom(:,:,t+1) = mom(:,:,t) + (tau * 2 / sigmaV2(t)) * dmom;

    dX(:) = 0;
    dmom(:) = 0;
end

iter = 0;
method = 'shooting';

clear dX dmom
savestruct('s',who)

    function kernelsum(t)
        if usefgt == 0
            for m = 1:nx
                for l = 1:nx
                    argin = -( ...
                        (X(1,m,t)-X(1,l,t))^2 + ...
                        (X(2,m,t)-X(2,l,t))^2 + ...
                        (X(3,m,t)-X(3,l,t))^2)/sigmaV2(t);
argout = exp(argin);  %% BUILT IN KERNEL kerV, do not remove this comment
                    dX(1,m) = dX(1,m) + argout * mom(1,l,t);
                    dX(2,m) = dX(2,m) + argout * mom(2,l,t);
                    dX(3,m) = dX(3,m) + argout * mom(3,l,t);
                end
            end
        else
            dX = dX + fgt(3,X(:,:,t),mom(:,:,t),X(:,:,t),sigmaV(t),order,K,e);
        end
    end

    function dkernelsum(t)
        if ~usefgt
            for m = 1:nx
                for l = 1:nx
                    argin = -( ...
                        (X(1,m,t)-X(1,l,t))^2 + ...
                        (X(2,m,t)-X(2,l,t))^2 + ...
                        (X(3,m,t)-X(3,l,t))^2)/sigmaV2(t);
argout = exp(argin);  %% BUILT IN KERNEL derV, do not remove this comment
                    argout = argout * (mom(1,l,t)*mom(1,m,t)+mom(2,l,t)*mom(2,m,t)+mom(3,l,t)*mom(3,m,t));
                    dmom(1,m) = dmom(1,m) + argout * (X(1,m,t)-X(1,l,t));
                    dmom(2,m) = dmom(2,m) + argout * (X(2,m,t)-X(2,l,t));
                    dmom(3,m) = dmom(3,m) + argout * (X(3,m,t)-X(3,l,t));
                end
            end
        else
            error('unfinished code: please set s.usefgt = 1')
            X = [X1,X2,X3]';
            sigmaV = sqrt(sigmaV2);

            resfgt = fgt([mom1,mom2,mom3,...
                mom1.*X1,mom1.*X2,mom1.*X3,...
                mom2.*X1,mom2.*X2,mom2.*X3,...
                mom3.*X1,mom3.*X2,mom3.*X3]'...
                ,X,X,sigmaV,order,K,e)';

            temp = mom1.*resfgt(:,1);
            dmom(:,1) = dmom(:,1) + temp.*X1;
            dmom(:,2) = dmom(:,2) + temp.*X2;
            dmom(:,3) = dmom(:,3) + temp.*X3;
            temp = mom2.*resfgt(:,2);
            dmom(:,1) = dmom(:,1) + temp.*X1;
            dmom(:,2) = dmom(:,2) + temp.*X2;
            dmom(:,3) = dmom(:,3) + temp.*X3;
            temp = mom3.*resfgt(:,3);
            dmom(:,1) = dmom(:,1) + temp.*X1;
            dmom(:,2) = dmom(:,2) + temp.*X2;
            dmom(:,3) = dmom(:,3) + temp.*X3;

            temp = mom1.*resfgt(:,4);
            dmom(:,1) = dmom(:,1) - temp;
            temp = mom1.*resfgt(:,5);
            dmom(:,2) = dmom(:,2) - temp;
            temp = mom1.*resfgt(:,6);
            dmom(:,3) = dmom(:,3) - temp;

            temp = mom2.*resfgt(:,7);
            dmom(:,1) = dmom(:,1) - temp;
            temp = mom2.*resfgt(:,8);
            dmom(:,2) = dmom(:,2) - temp;
            temp = mom2.*resfgt(:,9);
            dmom(:,3) = dmom(:,3) - temp;

            temp = mom3.*resfgt(:,10);
            dmom(:,1) = dmom(:,1) - temp;
            temp = mom3.*resfgt(:,11);
            dmom(:,2) = dmom(:,2) - temp;
            temp = mom3.*resfgt(:,12);
            dmom(:,3) = dmom(:,3) - temp;

        end

    end

end
